require "import"
import "android.widget.*"
import "com.androlua.*"
import "android.content.Context"
import "java.util.Calendar"
import "android.content.*"

local notes = {}
local folder = package.searchpath("main", package.path):match("(.*)main.lua")
local file_path = folder .. "notes.json"
local cjson = require "cjson"

-- Function to get current date and time as string
local function getCurrentDate()
  local cal = Calendar.getInstance()
  local year = tostring(cal.get(Calendar.YEAR))
  local month = tostring(cal.get(Calendar.MONTH) + 1)
  local day = tostring(cal.get(Calendar.DAY_OF_MONTH))
  local hour = tostring(cal.get(Calendar.HOUR_OF_DAY))
  local minute = tostring(cal.get(Calendar.MINUTE))
  local second = tostring(cal.get(Calendar.SECOND))
  return string.format("%04d-%02d-%02d %02d:%02d:%02d", year, month, day, hour, minute, second)
end

-- Function to load notes from the file
local function loadNotes()
  local file = io.open(file_path, "r")
  if file then
    local content = file:read("*a")
    file:close()
    if content ~= "" then
      notes = cjson.decode(content)
    end
  end
end

-- Function to save notes to the file
local function saveNotes()
  local file = io.open(file_path, "w")
  file:write(cjson.encode(notes))
  file:close()
end

-- Function to get titles of all notes
local function getTitles()
  local titles = {}
  for _, note in ipairs(notes) do
    local title = note.title
    if note.favorite then
      title = "★ " .. title
    end
    table.insert(titles, title .. " - " .. (note.date or ""))
  end
  return titles
end

-- Function to search notes based on keyword
local function searchNotes(key)
  local results = {}
  for _, note in ipairs(notes) do
    if note.title:lower():find(key:lower()) or note.note:lower():find(key:lower()) then
      table.insert(results, note)
    end
  end
  return results
end

-- Function to show the note dialog for adding or editing notes
local function showNoteDialog(oldTitle, oldNote, oldDate, editIndex)
  local layout = {
    LinearLayout,
    orientation = "vertical",
    {
      EditText,
      hint = "Title",
      id = "title_input",
      text = oldTitle or "",
    },
    {
      EditText,
      hint = "Enter your note...",
      id = "note_input",
      lines = 5,
      text = oldNote or "",
    }
  }

  local views = {}
  local dialog = LuaDialog(service)
  dialog.setTitle(editIndex and "Edit Note" or "Add Note")
  dialog.setView(loadlayout(layout, views))
  dialog.setButton("Save", function()
    local title = views.title_input.text
    local note = views.note_input.text
    if title == "" or note == "" then
      service.postSpeak(200, "Please fill in both the title and the note.")
      task(1, function()
        dialog.show()
      end)
      return
    end

    local date = oldDate or getCurrentDate()
    local isFav = note:lower():find("important") or title:lower():find("important")

    if editIndex then
      notes[editIndex] = {
        title = title,
        note = note,
        favorite = isFav and true or notes[editIndex].favorite,
        date = date
      }
    else
      table.insert(notes, {
        title = title,
        note = note,
        favorite = isFav and true or false,
        date = date
      })
    end

    saveNotes()
    list.adapter = ArrayAdapter(service, android.R.layout.simple_list_item_1, getTitles())
    task(1, function()
      main_dialog.show()
    end)
  end)

  dialog.setButton2("Cancel", function()
    dialog.dismiss()
    main_dialog.show()
  end)

  dialog.show()
end

-- Layout for the main dialog
local layout = {
  LinearLayout,
  orientation = "vertical",
  {
    EditText,
    id = "search_bar",
    hint = "Search notes...",
  },
  {
    ListView,
    id = "list",
  },
}

main_dialog = LuaDialog(service)
main_dialog.setTitle("My Notes")
main_dialog.setView(loadlayout(layout))

main_dialog.setButton2("About", function()
  local info = LuaDialog(service)
  info.setTitle("About This Extension")
  info.setMessage(
    "About this Extension:\n" ..
    "This is a powerful and simple notes-taking extension made specifically for Jieshuo screen reader users. It works smoothly with Jieshuo’s commentary service and does not require any additional apps.\n\n" ..

    "About the Developer:\n" ..
    "This extension was created by Rana Subhan. He is a passionate and skilled accessibility developer dedicated to building tools for visually impaired users.\n\n" ..

    "How to Use:\n" ..
    "- Tap 'New Note' to add a note.\n" ..
    "- Tap on a note to view it.\n" ..
    "- Long press on a note to see Share, Copy, Edit, Delete, Move to Top options.\n" ..
    "- Use the search bar to filter notes by title or content.\n\n" ..

    "This is not a standalone app. It is designed as a Jieshuo extension only."
  )
  info.setButton("OK", function()
    main_dialog.show()
  end)
  info.show()
end)

main_dialog.setButton3("New Note", function()
  task(50, function()
    showNoteDialog()
  end)
end)

main_dialog.setButton("Exit")

list.onItemClick = function(_, _, index, _)
  main_dialog.dismiss()
  local note = notes[index + 1]
  local d = LuaDialog(service)
  d.setTitle("Title: " .. note.title)
  local message = "Note:\n\n" .. note.note
  d.setMessage(message)
  d.setButton("OK", function()
    main_dialog.show()
  end)
  d.show()
end

list.onItemLongClick = function(_, _, index, _)
  local vibrator = service.getSystemService(Context.VIBRATOR_SERVICE)
  vibrator.vibrate(100)

  main_dialog.dismiss()
  local note = notes[index + 1]
  local menu = LuaDialog(service)
  menu.setTitle("Options")
  menu.setItems({"Share", "Copy", "Edit", "Delete", "Move to Top"})

  menu.onItemClick = function(_, _, selected)
    menu.dismiss()
    if selected == 0 then
      import "android.content.*"
      this.startActivity(Intent.createChooser(
        Intent(Intent.ACTION_SEND)
          .putExtra(Intent.EXTRA_TEXT, note.title .. "\n\n" .. note.note)
          .setType("text/plain"),
        nil
      ))
    elseif selected == 1 then
      local text = note.title .. "\n\n" .. note.note
      service.copy(text)
      service.postSpeak(200, "Notes Copied to clipboard")
      main_dialog.show()
    elseif selected == 2 then
      showNoteDialog(note.title, note.note, note.date, index + 1)
    elseif selected == 3 then
      local confirm = LuaDialog(service)
      confirm.setTitle("Delete Note")
      confirm.setMessage("Are you sure you want to delete this note?\n\n" .. note.title)
      confirm.setButton("Yes", function()
        table.remove(notes, index + 1)
        saveNotes()
        list.adapter = ArrayAdapter(service, android.R.layout.simple_list_item_1, getTitles())
        service.speak("Note deleted.")
        main_dialog.show()
      end)
      confirm.setButton2("No", function()
        main_dialog.show()
      end)
      confirm.show()
    elseif selected == 4 then
      note.favorite = true
      table.remove(notes, index + 1)
      table.insert(notes, 1, note)
      saveNotes()
      list.adapter = ArrayAdapter(service, android.R.layout.simple_list_item_1, getTitles())
      service.speak("Note moved to top.")
      main_dialog.show()
    end
  end

  menu.setButton("Cancel", function()
    main_dialog.show()
  end)
  menu.show()
end

search_bar.addTextChangedListener {
  onTextChanged = function(txt)
    local results = searchNotes(tostring(txt))
    local titles = {}
    for _, n in ipairs(results) do
      local title = n.title
      if n.favorite then
        title = "★ " .. title
      end
      table.insert(titles, title .. " - " .. (n.date or ""))
    end
    list.adapter = ArrayAdapter(service, android.R.layout.simple_list_item_1, titles)
  end
}

loadNotes()
list.adapter = ArrayAdapter(service, android.R.layout.simple_list_item_1, getTitles())
main_dialog.show()